@@ -1,5 +1,5 @@
 #! /bin/sh
-# Run this to generate all the initial makefiles, etc. 
+# Run this to generate all the initial makefiles, etc.
 #
 # Copyright (C) 2003 g10 Code GmbH
 #
@@ -42,7 +42,7 @@ w32ce_toolprefixes=
 w32ce_extraoptions=
 amd64_toolprefixes=
 # End list of optional variables sourced from ~/.gnupg-autogen.rc
-# What follows are variables which are sourced but default to 
+# What follows are variables which are sourced but default to
 # environment variables or lacking them hardcoded values.
 #w32root=
 #w32ce_root=
@@ -54,7 +54,7 @@ if [ -f "$HOME/.gnupg-autogen.rc" ]; then
 fi
 
 # Convenience option to use certain configure options for some hosts.
-myhost="" 
+myhost=""
 myhostsub=""
 case "$1" in
     --build-w32)
@@ -97,7 +97,7 @@ if [ "$myhost" = "w32" ]; then
           ;;
     esac
     echo "Using $w32root as standard install directory" >&2
-    
+
     crossbindir=
     for host in $toolprefixes; do
         if ${host}-gcc --version >/dev/null 2>&1 ; then
@@ -108,15 +108,15 @@ if [ "$myhost" = "w32" ]; then
     done
     if [ -z "$crossbindir" ]; then
         echo "Cross compiler kit not installed" >&2
-        if [ -z "$sub" ]; then 
+        if [ -z "$sub" ]; then
           echo "Under Debian GNU/Linux, you may install it using" >&2
-          echo "  apt-get install mingw32 mingw32-runtime mingw32-binutils" >&2 
+          echo "  apt-get install mingw32 mingw32-runtime mingw32-binutils" >&2
         fi
         echo "Stop." >&2
         exit 1
     fi
-   
-   
+
+
     if [ -f "$tsdir/config.log" ]; then
         if ! head $tsdir/config.log | grep "$host" >/dev/null; then
             echo "Pease run a 'make distclean' first" >&2
@@ -147,7 +147,7 @@ if test "$1" = "--build-amd64"; then
 
     [ -z "$amd64root" ] && amd64root="$HOME/amd64root"
     echo "Using $amd64root as standard install directory" >&2
-    
+
     # Locate the cross compiler
     crossbindir=
     for host in x86_64-linux-gnu amd64-linux-gnu; do
@@ -162,7 +162,7 @@ if test "$1" = "--build-amd64"; then
         echo "Stop." >&2
         exit 1
     fi
-   
+
     if [ -f "$tsdir/config.log" ]; then
         if ! head $tsdir/config.log | grep "$host" >/dev/null; then
             echo "Please run a 'make distclean' first" >&2
@@ -172,7 +172,7 @@ if test "$1" = "--build-amd64"; then
 
     $tsdir/configure --enable-maintainer-mode --prefix=${amd64root}  \
              --host=${host} --build=${build} \
-             --with-gpg-error-prefix=${amd64root} 
+             --with-gpg-error-prefix=${amd64root}
 
     rc=$?
     exit $rc
@@ -183,19 +183,19 @@ fi
 
 
 # Grep the required versions from configure.ac
-autoconf_vers=`sed -n '/^AC_PREREQ(/ { 
+autoconf_vers=`sed -n '/^AC_PREREQ(/ {
 s/^.*(\(.*\))/\1/p
 q
 }' ${configure_ac}`
 autoconf_vers_num=`echo "$autoconf_vers" | cvtver`
 
-automake_vers=`sed -n '/^min_automake_version=/ { 
+automake_vers=`sed -n '/^min_automake_version=/ {
 s/^.*="\(.*\)"/\1/p
 q
 }' ${configure_ac}`
 automake_vers_num=`echo "$automake_vers" | cvtver`
 
-#gettext_vers=`sed -n '/^AM_GNU_GETTEXT_VERSION(/ { 
+#gettext_vers=`sed -n '/^AM_GNU_GETTEXT_VERSION(/ {
 #s/^.*(\(.*\))/\1/p
 #q
 #}' ${configure_ac}`
@@ -234,13 +234,28 @@ fi
 if test "$DIE" = "yes"; then
     cat <<EOF
 
-Note that you may use alternative versions of the tools by setting 
+Note that you may use alternative versions of the tools by setting
 the corresponding environment variables; see README.CVS for details.
-                   
+
 EOF
     exit 1
 fi
 
+# Check the git setup.
+if [ -d .git ]; then
+  if [ -f .git/hooks/pre-commit.sample -a ! -f .git/hooks/pre-commit ] ; then
+    cat <<EOF >&2
+*** Activating trailing whitespace git pre-commit hook. ***
+    For more information see this thread:
+      http://mail.gnome.org/archives/desktop-devel-list/2009-May/msg00084html
+    To deactivate this pre-commit hook again move .git/hooks/pre-commit
+    and .git/hooks/pre-commit.sample out of the way.
+EOF
+      cp -av .git/hooks/pre-commit.sample .git/hooks/pre-commit
+      chmod -c +x  .git/hooks/pre-commit
+  fi
+fi
+
 echo "Running aclocal -I m4 -I gl/m4 ${ACLOCAL_FLAGS:+$ACLOCAL_FLAGS }..."
 $ACLOCAL -I m4 -I gl/m4 $ACLOCAL_FLAGS
 echo "Running autoheader..."
