@@ -20,7 +20,7 @@
 #include <config.h>
 #include <stdarg.h>
 
-#define _KSBA_INCLUDED_BY_VISIBILITY_C 
+#define _KSBA_INCLUDED_BY_VISIBILITY_C
 #include "util.h"
 
 /*--version.c --*/
@@ -32,7 +32,7 @@ ksba_check_version (const char *req_version)
 
 
 /*-- util.c --*/
-void 
+void
 ksba_set_malloc_hooks ( void *(*new_alloc_func)(size_t n),
                         void *(*new_realloc_func)(void *p, size_t n),
                         void (*new_free_func)(void*) )
@@ -41,7 +41,7 @@ ksba_set_malloc_hooks ( void *(*new_alloc_func)(size_t n),
 }
 
 
-void 
+void
 ksba_set_hash_buffer_function ( gpg_error_t (*fnc)
                                 (void *arg, const char *oid,
                                  const void *buffer, size_t length,
@@ -86,7 +86,7 @@ ksba_free ( void *a )
 
 
 /*-- cert.c --*/
-gpg_error_t 
+gpg_error_t
 ksba_cert_new (ksba_cert_t *acert)
 {
   return _ksba_cert_new (acert);
@@ -152,7 +152,7 @@ ksba_cert_hash (ksba_cert_t cert,
                 int what,
                 void (*hasher)(void *,
                                const void *,
-                               size_t length), 
+                               size_t length),
                 void *hasher_arg)
 {
   return _ksba_cert_hash (cert, what, hasher, hasher_arg);
@@ -180,7 +180,7 @@ ksba_cert_get_issuer (ksba_cert_t cert, int idx)
 }
 
 
-gpg_error_t 
+gpg_error_t
 ksba_cert_get_validity (ksba_cert_t cert, int what,
                         ksba_isotime_t r_time)
 {
@@ -195,7 +195,7 @@ ksba_cert_get_subject (ksba_cert_t cert, int idx)
 }
 
 
-ksba_sexp_t 
+ksba_sexp_t
 ksba_cert_get_public_key (ksba_cert_t cert)
 {
   return _ksba_cert_get_public_key (cert);
@@ -210,12 +210,12 @@ ksba_cert_get_sig_val (ksba_cert_t cert)
 
 
 
-gpg_error_t 
+gpg_error_t
 ksba_cert_get_extension (ksba_cert_t cert, int idx,
                          char const **r_oid, int *r_crit,
                          size_t *r_deroff, size_t *r_derlen)
 {
-  return _ksba_cert_get_extension (cert, idx, r_oid, r_crit, 
+  return _ksba_cert_get_extension (cert, idx, r_oid, r_crit,
                                    r_deroff, r_derlen);
 }
 
@@ -249,7 +249,7 @@ ksba_cert_get_ext_key_usages (ksba_cert_t cert, char **result)
 }
 
 
-gpg_error_t 
+gpg_error_t
 ksba_cert_get_crl_dist_point (ksba_cert_t cert, int idx,
                               ksba_name_t *r_distpoint,
                               ksba_name_t *r_issuer,
@@ -284,7 +284,7 @@ ksba_cert_get_authority_info_access (ksba_cert_t cert, int idx,
                                      char **r_method,
                                      ksba_name_t *r_location)
 {
-  return _ksba_cert_get_authority_info_access (cert, idx, 
+  return _ksba_cert_get_authority_info_access (cert, idx,
                                                r_method, r_location);
 }
 
@@ -319,7 +319,7 @@ ksba_cms_new (ksba_cms_t *r_cms)
 void
 ksba_cms_release (ksba_cms_t cms)
 {
-  return _ksba_cms_release (cms);
+  _ksba_cms_release (cms);
 }
 
 
@@ -441,7 +441,7 @@ ksba_cms_set_hash_function (ksba_cms_t cms,
                             void (*hash_fnc)(void *, const void *, size_t),
                             void *hash_fnc_arg)
 {
-  return _ksba_cms_set_hash_function (cms, hash_fnc, hash_fnc_arg);
+  _ksba_cms_set_hash_function (cms, hash_fnc, hash_fnc_arg);
 }
 
 
@@ -555,7 +555,7 @@ ksba_crl_new (ksba_crl_t *r_crl)
 void
 ksba_crl_release (ksba_crl_t crl)
 {
-  return _ksba_crl_release (crl);
+  _ksba_crl_release (crl);
 }
 
 
@@ -572,7 +572,7 @@ ksba_crl_set_hash_function (ksba_crl_t crl,
                                              const void *, size_t),
                             void *hash_fnc_arg)
 {
-  return _ksba_crl_set_hash_function (crl, hash_fnc, hash_fnc_arg);
+  _ksba_crl_set_hash_function (crl, hash_fnc, hash_fnc_arg);
 }
 
 
@@ -591,7 +591,7 @@ ksba_crl_get_issuer (ksba_crl_t crl, char **r_issuer)
 
 
 gpg_error_t
-ksba_crl_get_extension (ksba_crl_t crl, int idx, 
+ksba_crl_get_extension (ksba_crl_t crl, int idx,
                         char const **oid, int *critical,
                         unsigned char const **der, size_t *derlen)
 {
@@ -662,7 +662,7 @@ ksba_ocsp_new (ksba_ocsp_t *r_oscp)
 void
 ksba_ocsp_release (ksba_ocsp_t ocsp)
 {
-  return _ksba_ocsp_release (ocsp);
+  _ksba_ocsp_release (ocsp);
 }
 
 
@@ -707,7 +707,7 @@ ksba_ocsp_prepare_request (ksba_ocsp_t ocsp)
 gpg_error_t
 ksba_ocsp_hash_request (ksba_ocsp_t ocsp,
                         void (*hasher)(void *, const void *,
-                                       size_t length), 
+                                       size_t length),
                         void *hasher_arg)
 {
   return _ksba_ocsp_hash_request (ocsp, hasher, hasher_arg);
@@ -760,7 +760,7 @@ gpg_error_t
 ksba_ocsp_hash_response (ksba_ocsp_t ocsp,
                          const unsigned char *msg, size_t msglen,
                          void (*hasher)(void *, const void *,
-                                        size_t length), 
+                                        size_t length),
                          void *hasher_arg)
 {
   return _ksba_ocsp_hash_response (ocsp, msg, msglen, hasher, hasher_arg);
@@ -808,10 +808,10 @@ gpg_error_t
 ksba_ocsp_get_extension (ksba_ocsp_t ocsp, ksba_cert_t cert,
                          int idx,
                          char const **r_oid, int *r_crit,
-                         unsigned char const **r_der, 
+                         unsigned char const **r_der,
                          size_t *r_derlen)
 {
-  return _ksba_ocsp_get_extension (ocsp, cert, idx, r_oid, r_crit, 
+  return _ksba_ocsp_get_extension (ocsp, cert, idx, r_oid, r_crit,
                                    r_der, r_derlen);
 }
 
@@ -829,7 +829,7 @@ ksba_certreq_new (ksba_certreq_t *r_cr)
 void
 ksba_certreq_release (ksba_certreq_t cr)
 {
-  return _ksba_certreq_release (cr);
+  _ksba_certreq_release (cr);
 }
 
 
@@ -845,7 +845,21 @@ ksba_certreq_set_hash_function (ksba_certreq_t cr,
                                 void (*hash_fnc)(void *, const void *, size_t),
                                 void *hash_fnc_arg)
 {
-  return _ksba_certreq_set_hash_function (cr, hash_fnc, hash_fnc_arg);
+  _ksba_certreq_set_hash_function (cr, hash_fnc, hash_fnc_arg);
+}
+
+
+gpg_error_t
+ksba_certreq_set_serial (ksba_certreq_t cr, ksba_const_sexp_t sn)
+{
+  return _ksba_certreq_set_serial (cr, sn);
+}
+
+
+gpg_error_t
+ksba_certreq_set_issuer (ksba_certreq_t cr, const char *name)
+{
+  return _ksba_certreq_set_issuer (cr, name);
 }
 
 
@@ -890,6 +904,19 @@ ksba_certreq_build (ksba_certreq_t cr,
 }
 
 
+gpg_error_t
+ksba_certreq_set_validity (ksba_certreq_t cr, int what,
+                           const ksba_isotime_t timebuf)
+{
+  return _ksba_certreq_set_validity (cr, what, timebuf);
+}
+
+
+gpg_error_t
+ksba_certreq_set_siginfo (ksba_certreq_t cr, ksba_const_sexp_t siginfo)
+{
+  return _ksba_certreq_set_siginfo (cr, siginfo);
+}
 
 
 /*-- reader.c --*/
@@ -903,7 +930,7 @@ ksba_reader_new (ksba_reader_t *r_r)
 void
 ksba_reader_release (ksba_reader_t r)
 {
-  return _ksba_reader_release (r);
+  _ksba_reader_release (r);
 }
 
 
@@ -946,7 +973,7 @@ ksba_reader_set_file (ksba_reader_t r, FILE *fp)
 
 
 gpg_error_t
-ksba_reader_set_cb (ksba_reader_t r, 
+ksba_reader_set_cb (ksba_reader_t r,
                     int (*cb)(void*,char *,size_t,size_t*),
                     void *cb_value )
 {
@@ -989,7 +1016,7 @@ ksba_writer_new (ksba_writer_t *r_w)
 void
 ksba_writer_release (ksba_writer_t w)
 {
-  return _ksba_writer_release (w);
+  _ksba_writer_release (w);
 }
 
 
@@ -1022,7 +1049,7 @@ ksba_writer_set_file (ksba_writer_t w, FILE *fp)
 
 
 gpg_error_t
-ksba_writer_set_cb (ksba_writer_t w, 
+ksba_writer_set_cb (ksba_writer_t w,
                     int (*cb)(void*,const void *,size_t),
                     void *cb_value)
 {
@@ -1052,7 +1079,7 @@ ksba_writer_snatch_mem (ksba_writer_t w, size_t *nbytes)
 
 
 gpg_error_t
-ksba_writer_set_filter (ksba_writer_t w, 
+ksba_writer_set_filter (ksba_writer_t w,
                         gpg_error_t (*filter)(void*,
                                               const void *,size_t, size_t *,
                                               void *, size_t, size_t *),
@@ -1081,7 +1108,7 @@ ksba_writer_write_octet_string (ksba_writer_t w,
 
 
 /*-- asn1-parse.y --*/
-int 
+int
 ksba_asn_parse_file (const char *filename, ksba_asn_tree_t *result,
                      int debug)
 {
@@ -1092,7 +1119,7 @@ ksba_asn_parse_file (const char *filename, ksba_asn_tree_t *result,
 void
 ksba_asn_tree_release (ksba_asn_tree_t tree)
 {
-  return _ksba_asn_tree_release (tree);
+  _ksba_asn_tree_release (tree);
 }
 
 
@@ -1100,7 +1127,7 @@ ksba_asn_tree_release (ksba_asn_tree_t tree)
 void
 ksba_asn_tree_dump (ksba_asn_tree_t tree, const char *name, FILE *fp)
 {
-  return _ksba_asn_tree_dump (tree, name, fp);
+  _ksba_asn_tree_dump (tree, name, fp);
 }
 
 
@@ -1156,7 +1183,7 @@ ksba_dn_str2der (const char *string,
 
 
 gpg_error_t
-ksba_dn_teststr (const char *string, int seq, 
+ksba_dn_teststr (const char *string, int seq,
                  size_t *rerroff, size_t *rerrlen)
 {
   return _ksba_dn_teststr (string, seq, rerroff, rerrlen);
@@ -1176,14 +1203,14 @@ ksba_name_new (ksba_name_t *r_name)
 void
 ksba_name_ref (ksba_name_t name)
 {
-  return _ksba_name_ref (name);
+  _ksba_name_ref (name);
 }
 
 
 void
 ksba_name_release (ksba_name_t name)
 {
-  return _ksba_name_release (name);
+  _ksba_name_release (name);
 }
 
 
@@ -1199,5 +1226,3 @@ ksba_name_get_uri (ksba_name_t name, int idx)
 {
   return _ksba_name_get_uri (name, idx);
 }
-
-
